package org.eclipse.ecf.provider.ftp.container;

import java.io.IOException;
import java.util.HashMap;
import javax.security.auth.callback.Callback;
import javax.security.auth.callback.CallbackHandler;
import org.apache.commons.net.ftp.FTPClient;
import org.apache.commons.net.ftp.FTPReply;
import org.eclipse.ecf.core.ContainerConnectException;
import org.eclipse.ecf.core.ISharedObjectContainerConfig;
import org.eclipse.ecf.core.SharedObjectAddException;
import org.eclipse.ecf.core.comm.ConnectionInstantiationException;
import org.eclipse.ecf.core.comm.ISynchAsynchConnection;
import org.eclipse.ecf.core.identity.ID;
import org.eclipse.ecf.core.identity.IDFactory;
import org.eclipse.ecf.core.identity.IDInstantiationException;
import org.eclipse.ecf.core.identity.Namespace;
import org.eclipse.ecf.core.security.IConnectContext;
import org.eclipse.ecf.core.security.ObjectCallback;
import org.eclipse.ecf.fileshare.IFileShareContainer;
import org.eclipse.ecf.fileshare.IRetrieveFileDescription;
import org.eclipse.ecf.fileshare.ISharedFile;
import org.eclipse.ecf.fileshare.IStoreFileDescription;
import org.eclipse.ecf.fileshare.SharedFileCreationException;
import org.eclipse.ecf.provider.ftp.FtpPlugin;
import org.eclipse.ecf.provider.ftp.identity.FtpID;
import org.eclipse.ecf.provider.generic.ClientSOContainer;


public class FtpClientSOContainer extends ClientSOContainer implements
		IFileShareContainer {
	
	FTPClient ftpClient = null;
	
	public FtpClientSOContainer(ISharedObjectContainerConfig config) {
		super(config);
	}
	public void connect(ID targetID, IConnectContext connectContext) throws ContainerConnectException {
		if (!(targetID instanceof FtpID)) throw new ContainerConnectException("targetID "+targetID+" not appropriate type");
		FtpID target = (FtpID) targetID;
		String host = target.getHost();
		int port = target.getPort();
		String user = target.getUserInfo();
		String path = target.getPath();
		String password = null;
		if (connectContext != null) {
			CallbackHandler ch = connectContext.getCallbackHandler();
			ObjectCallback oc = new ObjectCallback();
			try {
				ch.handle(new Callback[] { oc });
				password = (String) oc.getObject();
			} catch (Exception e) {
				throw new ContainerConnectException("callback handler received unexpected exception",e);
			}
		}
		// Now create client and connect
		ftpClient = new FTPClient();
		try {
			ftpClient.connect(host,port);
			int replyCode = ftpClient.getReplyCode();
	        if (!FTPReply.isPositiveCompletion(replyCode)) throw new IOException("connect failed.  code="+replyCode);
		} catch (IOException e) {
			disconnect();
			throw new ContainerConnectException("connect exception connecting to "+target,e);
		}
		// login
		try {
			boolean login = ftpClient.login(user,password);
			if (!login) throw new IOException("login to "+target+" not successful");
		} catch (IOException e) {
			disconnect();
			throw new ContainerConnectException("exception in login to "+target,e);
		}
		// change to path
		if (path != null && (path.length() > 0)) {
			try {
				boolean result = ftpClient.changeWorkingDirectory(path);
				if (!result) throw new IOException("change to working directory "+path+" not successful");
			} catch (IOException e) {
				disconnect();
				throw new ContainerConnectException("exception changing working directory to "+path,e);
			}
		}
		this.remoteServerID = target;
		// We're done
	}
	public void disconnect() {
		if (ftpClient != null) {
			try {
				ftpClient.disconnect();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			ftpClient = null;
		}
	}
	protected ISynchAsynchConnection makeConnection(ID remoteSpace, Object data)
			throws ConnectionInstantiationException {
		return null;
	}
	public Namespace getConnectNamespace() {
		return IDFactory.getDefault().getNamespaceByName(FtpPlugin.NAMESPACE_NAME);
	}
	public Object getAdapter(Class intf) {
		if (intf.equals(IFileShareContainer.class)) return this;
		else return null;
	}
	public ISharedFile store(IStoreFileDescription spec) throws SharedFileCreationException {
		FtpStoreSharedObject ftpstore = new FtpStoreSharedObject(ftpClient,spec);
		try {
			getSharedObjectManager().addSharedObject(IDFactory.getDefault().makeGUID(),ftpstore,new HashMap());
		} catch (SharedObjectAddException e) {
			throw new SharedFileCreationException("file object creation exception",e);
		} catch (IDInstantiationException e) {
			throw new SharedFileCreationException("file object id creation exception",e);
		}
		return ftpstore;
	}
	public ISharedFile retrieve(IRetrieveFileDescription spec) throws SharedFileCreationException {
		// TODO Auto-generated method stub
		return null;
	}
}
