package org.eclipse.ecf.provider.ftp.identity;

import java.net.URI;
import java.net.URISyntaxException;
import org.eclipse.ecf.core.identity.BaseID;
import org.eclipse.ecf.core.identity.IDInstantiationException;
import org.eclipse.ecf.core.identity.Namespace;

public class FtpID extends BaseID {
	private static final long serialVersionUID = 1713749834601456262L;
	
	public static final int FTP_PORT = 21;
	public static final String ANONYMOUS_USER = "anonymous";
	public static final String FTP_SCHEME = "ftp:";
	
	URI ftpURI;
	
	public FtpID() {
		super();
	}
	protected FtpID(Namespace namespace) {
		super(namespace);
	}
	public FtpID(Namespace namespace, URI uri) throws IDInstantiationException {
		super(namespace);
		this.ftpURI = uri;
		String scheme = uri.getScheme();
		if (scheme.equalsIgnoreCase(FTP_SCHEME)) throw new IDInstantiationException("not ftp scheme in uri "+uri);
	}
	protected int namespaceCompareTo(BaseID o) {
		return getName().compareTo(o.getName());
	}
	protected boolean namespaceEquals(BaseID o) {
		if (!(o instanceof FtpID)) {
			return false;
		}
		FtpID other = (FtpID) o;
		return other.ftpURI.equals(ftpURI);
	}
	protected String namespaceGetName() {
		return ftpURI.toString();
	}
	protected int namespaceHashCode() {
		return ftpURI.hashCode();
	}
	protected URI namespaceToURI() throws URISyntaxException {
		return ftpURI;
	}
	public String getHost() {
		return ftpURI.getHost();
	}
	public String getUserInfo() {
		String user = ftpURI.getUserInfo();
		if (user == null) return ANONYMOUS_USER;
		else return user;
	}
	public int getPort() {
		int p = ftpURI.getPort();
		if (p == -1) return FTP_PORT;
		else return p;
	}
	protected String relativizePath(String path) {
		if (path == null) return null;
		String rPath = path;
		while(rPath.startsWith("/")) {
			rPath = rPath.substring(1,rPath.length());
		}
		return rPath;
	}
	public String getPath() {
		return relativizePath(ftpURI.getPath());
	}
}
